# yaylib
# Copyright 2023 qvco
# See LICENSE for details.

import os
import json
import unittest


# import yaylib

test_email = os.environ.get("TEST_EMAIL", "")
test_password = os.environ.get("TEST_PASSWORD", "")
